package de.hhhammer.prozessor.jsb;


import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;

class JMethodTest {

    @Test
    void should() {
        final var method = new JMethod<>(JAccessor.PUBLIC, JModifier.FINAL, new IntegerType(), new MethodName(), new MethodLogic());

        final var expect = """
                public final Integer methodFoo(Integer arg0, Integer arg1) {
                \treturn arg0 + arg1;
                }
                """;

        assertEquals(expect, method.represent());
    }

    static class IntegerType implements Type<Integer> {

        @Override
        public String representImports() {
            return "";
        }

        @Override
        public String represent() {
            return Integer.class.getSimpleName();
        }

    }

    static class IntegerField implements Field<Integer> {
        private final Integer value;

        IntegerField(final Integer value) {
            this.value = value;
        }

        @Override
        public Integer getValue() {
            return value;
        }

        @Override
        public String representImports() {
            return "";
        }

        @Override
        public String represent() {
            return Integer.class.getSimpleName();
        }
    }

    static class MethodName implements Name {

        @Override
        public String represent() {
            return "methodFoo";
        }
    }

    static class MethodLogic implements ExpressionWithArguments<IntegerType> {

        private final IntegerField arg0 = new IntegerField(1);
        private final IntegerField arg1 = new IntegerField(2);

        @Override
        public String representArguments() {
            return arg0.represent() + " arg0, " + arg1.represent() + " arg1";
        }

        @Override
        public String representExpression() {
            return "return arg0 + arg1;";
        }

        @Override
        public String representImports() {
            return "";
        }
    }
}